/*
 * Copyright 2015 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "service.h"

#include <QByteArray>
#include <QDebug>
#include <QDir>
#include <QJsonArray>
#include <QJsonDocument>
#include <QStandardPaths>

using namespace CookieService;

namespace CookieService {

class ServicePrivate
{
    Q_DECLARE_PUBLIC(Service)

public:
    ServicePrivate(Service *service);
    ~ServicePrivate();

    void setIdle(bool idle);

private:
    QDir m_baseDir;
    bool m_idle;
    mutable Service *q_ptr;
};

} // namespace

ServicePrivate::ServicePrivate(Service *q):
    m_idle(true),
    q_ptr(q)
{
    QString basePath =
        QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation) +
        QStringLiteral("/webapp-cookie-service");
    m_baseDir = QDir(basePath);
    if (!m_baseDir.exists()) {
        m_baseDir.mkpath(".");
    }
}

ServicePrivate::~ServicePrivate()
{
}

void ServicePrivate::setIdle(bool idle)
{
    Q_Q(Service);

    if (idle != m_idle) {
        m_idle = idle;
        Q_EMIT q->isIdleChanged();
    }
}

Service::Service(QObject *parent):
    QObject(parent),
    d_ptr(new ServicePrivate(this))
{
}

Service::~Service()
{
    delete d_ptr;
}

bool Service::isIdle() const
{
    Q_D(const Service);
    return d->m_idle;
}

void Service::saveCookies(const QString &path, const Cookies &cookies)
{
    Q_D(Service);

    d->setIdle(false);
    qDebug() << "Saving cookies for" << path;

    QJsonDocument doc = QJsonDocument::fromJson(cookies.toUtf8());

    QFile file(d->m_baseDir.filePath(path));
    if (Q_LIKELY(file.open(QIODevice::WriteOnly | QIODevice::Text))) {
        file.write(doc.toJson());
        file.close();
    }

    d->setIdle(true);
}

Cookies Service::loadCookies(const QString &path)
{
    Q_D(Service);

    d->setIdle(false);
    qDebug() << "Loading cookies for" << path;

    Cookies cookies;

    QFile file(d->m_baseDir.filePath(path));
    if (Q_UNLIKELY(!file.open(QIODevice::ReadOnly | QIODevice::Text))) {
        qWarning() << "Cannot open file" << file.fileName();
        return cookies;
    }

    QByteArray contents = file.readAll();
    QJsonDocument doc = QJsonDocument::fromJson(contents);
    if (doc.isEmpty() || !doc.isArray()) return cookies;

    cookies = QString::fromUtf8(doc.toJson(QJsonDocument::Compact));
    d->setIdle(true);
    return cookies;
}
