/*
 * Copyright 2015 Canonical Ltd.
 *
 * This file is part of webbrowser-app.
 *
 * webbrowser-app is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * webbrowser-app is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "shared-cookie-store.h"

#include <QByteArray>
#include <QDBusConnection>
#include <QDBusMessage>
#include <QDBusMetaType>
#include <QDebug>
#include <QFile>
#include <QFileInfo>
#include <QJsonArray>
#include <QJsonDocument>
#include <QNetworkCookie>
#include <QStandardPaths>
#include "oxide-cookie-helper.h"

#define COOKIE_SERVICE_SERVICE_NAME "com.canonical.cookie-sharing"
#define COOKIE_SERVICE_OBJECT_PATH "/"
#define COOKIE_SERVICE_INTERFACE COOKIE_SERVICE_SERVICE_NAME

class SharedCookieStorePrivate
{
public:
    SharedCookieStorePrivate():
        m_connection(QDBusConnection::sessionBus())
    {}

    QString m_dbPath;
    QDBusConnection m_connection;
};

static QList<QNetworkCookie> cookiesFromVariant(const QVariantList &cl)
{
    QList<QNetworkCookie> cookies;
    Q_FOREACH(QVariant cookie, cl) {
        if (!cookie.canConvert(QVariant::Map)) {
            continue;
        }

        QNetworkCookie nc;
        QVariantMap vm = cookie.toMap();
        if (!vm.contains("name") || !vm.contains("value")) {
            continue;
        }

        nc.setName(vm.value("name").toByteArray());
        nc.setValue(vm.value("value").toByteArray());
        nc.setDomain(vm.value("domain").toString());
        nc.setPath(vm.value("path").toString());
        if (vm.contains("httponly") &&
            vm.value("httponly").canConvert(QVariant::Bool)) {
            nc.setHttpOnly(vm.value("httponly").toBool());
        }

        if (vm.contains("issecure") &&
            vm.value("issecure").canConvert(QVariant::Bool)) {
            nc.setSecure(vm.value("issecure").toBool());
        }

        if (vm.contains("expirationdate") &&
            vm.value("expirationdate").canConvert(QMetaType::QDateTime)) {
            nc.setExpirationDate(vm.value("expirationdate").toDateTime());
        }

        cookies.append(nc);
    }
    return cookies;
}

SharedCookieStore::SharedCookieStore(QObject* parent):
    CookieStore(parent)
{
}

void SharedCookieStore::doGetCookies()
{
    Q_D(const SharedCookieStore);

    QDBusMessage message =
        QDBusMessage::createMethodCall(COOKIE_SERVICE_SERVICE_NAME,
                                       COOKIE_SERVICE_OBJECT_PATH,
                                       COOKIE_SERVICE_INTERFACE,
                                       "loadCookies");
    QVariantList args;
    args << d->m_dbPath;
    message.setArguments(args);

    QDBusMessage reply = d->m_connection.call(message);
    if (reply.type() == QDBusMessage::ErrorMessage) {
        qWarning() << "Got error:" << reply.errorMessage();
        Q_EMIT gotCookies(Cookies());
        return;
    }

    Cookies cookies;
    args = reply.arguments();
    if (Q_UNLIKELY(args.isEmpty())) {
        qWarning() << "No cookies in reply!";
        Q_EMIT gotCookies(cookies);
        return;
    }

    QString jsonCookies = args.at(0).toString();
    QJsonDocument doc = QJsonDocument::fromJson(jsonCookies.toUtf8());
    QVariantList cookieVariants = doc.array().toVariantList();

    Q_EMIT gotCookies(cookiesFromVariant(cookieVariants));
}

QDateTime SharedCookieStore::lastUpdateTimeStamp() const
{
    return QDateTime();
}

void SharedCookieStore::doSetCookies(const Cookies& parsedCookies)
{
    Q_D(const SharedCookieStore);
    QVariant cookies = OxideCookieHelper::variantFromCookies(parsedCookies);
    QJsonArray jsonCookies =
        QJsonArray::fromVariantList(cookies.toList());
    QJsonDocument doc(jsonCookies);

    QDBusMessage message =
        QDBusMessage::createMethodCall(COOKIE_SERVICE_SERVICE_NAME,
                                       COOKIE_SERVICE_OBJECT_PATH,
                                       COOKIE_SERVICE_INTERFACE,
                                       "saveCookies");
    QVariantList args;
    args << d->m_dbPath;
    args << QString::fromUtf8(doc.toJson(QJsonDocument::Compact));
    message.setArguments(args);

    QDBusMessage reply = d->m_connection.call(message);
    Q_EMIT cookiesSet(reply.type() == QDBusMessage::ReplyMessage);
}

void SharedCookieStore::setDbPath(const QString &path)
{
    Q_D(SharedCookieStore);
    if (path != d->m_dbPath) {
        d->m_dbPath = path;
        Q_EMIT dbPathChanged();
    }
}

QString SharedCookieStore::dbPath() const
{
    Q_D(const SharedCookieStore);
    return d->m_dbPath;
}
